package com.alibaba.opensec.sample;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.TreeMap;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

public abstract class BaseTest {
	
	protected String appkey = "";
	
	protected String secret = "";
	
	protected String topAppkey = ""; // 如果没有TOP appkey，可以不填，样例代码中会使用御城河appkey代替topAppkey
	
	protected String apiHost = "http://gw.ose.aliyun.com/event/"; // 日志接入API
	
	protected String accountHost = "http://account.ose.aliyun.com/"; // 账号风控API
	
	protected Map<String, String> defaultParam = new TreeMap<String, String>();
	
	public BaseTest() throws Exception {
		if (appkey.isEmpty() || secret.isEmpty()) {
    		throw new Exception("appkey or secret is empty.");
    	}
    	
    	if (topAppkey.isEmpty()) {
    		topAppkey = appkey;
    	}
    	
    	SimpleDateFormat sFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date now = new Date();
		String time = sFormat.format(now);

		defaultParam.put("appName", "我的测试应用");
		defaultParam.put("time", time);
		defaultParam.put("ntime", new Long(now.getTime()).toString());
		defaultParam.put("ati", "123456789123456789");
		defaultParam.put("userId", "tester");
		defaultParam.put("userIp", "42.120.74.9");
	}
	
	protected void parseResult(String rsponse) {
    	String result = "";	
		String errMsg = "";
		
		if (rsponse != null) {
			JSONObject obj = null;
			
			try {
				obj = (JSONObject) JSON.parse(rsponse);
			} catch (Exception e) {			
			}
			
			if (obj != null) {
				result = obj.getString("result");
				errMsg = obj.getString("errMsg");
			}
		}
		
		if ("success".equals(result) && (errMsg == null || errMsg.isEmpty())) {
			System.out.println("[" + getShortClassName() + "] Success.");
		} else {
			System.out.println("[" + getShortClassName() + "] Error:" + rsponse);
		}
    }
	
	private String getShortClassName() {
		String name = getClass().getName();
		int pos = name.lastIndexOf(".");
		
		if (pos >= 0) {
			name = name.substring(pos + 1);
		}
		
		return name;
	}
	
	public abstract void exec();
}
